from __future__ import absolute_import

import datetime
import logging
from operator import itemgetter

from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.sites.models import Site

from .util import ChoiceEnum
from . import settings as local_settings
try:
    from positions.fields import PositionField
except ImportError:
    logging.warn('positions not installed. '
                 'Will just use integers for position fields.')
    PositionField = None


class LiveEmailFormManager(models.Manager):

    def get_query_set(self):
        now = datetime.datetime.now()
        return super(LiveEmailFormManager, self).get_query_set().filter(
            is_published=True,
            starts_at__lte=now)


class EmailForm(models.Model):

    class Meta:
        ordering = ('-starts_at',)

    title = models.CharField(max_length=80)
    slug = models.SlugField()
    description = models.TextField(blank=True)
    starts_at = models.DateTimeField(default=datetime.datetime.now)
    is_published = models.BooleanField(default=False)
    thanks_message = models.TextField(default=_("Thanks for entering!"))
    email = models.CharField(
        blank=True,
        help_text=_("Send a notification to this email whenever someone "
                   "submits an entry to this email form",),
        max_length=300)
    site = models.ForeignKey(Site)

    class Meta:
        unique_together = (('slug', 'site'),)

    def __unicode__(self):
        return self.title

    def get_url_path(self):
        kwargs={'slug' : self.slug}
        return reverse('emailform_detail',
                       kwargs=kwargs)

    def get_full_url(self):
        url = self.get_url_path()
        site = self.site or Site.objects.get_current()
        return 'http://%s%s' % (site.domain, url)

    def get_absolute_url(self):
        return self.get_full_url()

    objects = models.Manager()
    live = LiveEmailFormManager()

OPTION_TYPE_CHOICES = ChoiceEnum(sorted([('char', 'Text Field'),
                                         ('email', 'Email Field'),
                                         ('file', 'File Upload'),
                                         ('integer', 'Integer'),
                                         ('float', 'Float'),
                                         ('bool', 'Boolean'),
                                         ('text', 'Text Area'),
                                         ('select', 'Select One Choice'),
                                         ('radio', 'Radio List'),
                                         ('checkbox', 'Checkbox List'),
                                         ('zipcode', 'Zipcode'),
                                         ('usstate', 'US State'),
                                         ('phonenumber', 'Phone Number')],
                                        key=itemgetter(1)))


class Question(models.Model):

    class Meta:
        ordering = ('order',)

    emailform = models.ForeignKey(EmailForm, related_name="questions")
    question = models.TextField(help_text=_(
        "Appears on the email form entry page."))
    label = models.CharField(max_length=32, help_text=_(
        "Appears on the results page."))
    help_text = models.TextField(blank=True)
    required = models.BooleanField(default=False)
    if PositionField:
        order = PositionField(collection=('emailform',))
    else:
        order = models.IntegerField()
    option_type = models.CharField(max_length=12, choices=OPTION_TYPE_CHOICES)
    options = models.TextField(blank=True, default='')

    def __unicode(self):
        return self.question

    @property
    def parsed_options(self):
        if OPTION_TYPE_CHOICES.BOOLEAN == self.option_type:
            return [True, False]
        return filter(None, (s.strip() for s in self.options.splitlines()))


class Entry(models.Model):

    class Meta:
        verbose_name = "Entry"
        verbose_name_plural = "Entries"
        ordering = ('-submitted_at',)

    emailform = models.ForeignKey(EmailForm)
    ip_address = models.IPAddressField()
    submitted_at = models.DateTimeField(default=datetime.datetime.now)
    content = models.TextField(blank=True)



from __future__ import absolute_import

import re

from django.contrib import admin
from django.forms import ModelForm, ValidationError

from .models import Question, EmailForm, Entry


class QuestionInline(admin.StackedInline):
    model = Question
    extra = 3


class EmailFormAdminForm(ModelForm):

    class Meta:
        model = EmailForm


class EmailFormAdmin(admin.ModelAdmin):
    save_as = True
    form = EmailFormAdminForm
    search_fields = ('title', 'slug', 'description')
    prepopulated_fields = {'slug': ('title',)}
    list_display = (
        'title',
        'slug',
        'is_published',
        'site')
    list_filter = ('is_published', 'site')
    date_hierarchy = 'starts_at'
    inlines = [QuestionInline]


admin.site.register(EmailForm, EmailFormAdmin)


class EntryAdmin(admin.ModelAdmin):
    search_fields = ('content',)
    list_display = ('emailform', 'submitted_at',
                    'ip_address', )
    list_filter = ('emailform', 'submitted_at', )
    date_hierarchy = 'submitted_at'


admin.site.register(Entry, EntryAdmin)



from __future__ import absolute_import

from django.conf.urls.defaults import patterns, url

from .views import emailform_detail

urlpatterns = patterns(
    "",
    url(r'^(?P<slug>[-a-z0-9_]+)/embed/$',
        emailform_detail,
        {'embed' : True},
        name="emailform_detail"),

    url(r'^(?P<slug>[-a-z0-9_]+)/$',
        emailform_detail,
        name="emailform_detail"),
    )



from __future__ import absolute_import

import logging

from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.core.urlresolvers import reverse, NoReverseMatch
from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext as _rc
from django.template.loader import render_to_string

from .forms import forms_for_emailform, BaseOptionAnswer
from .models import EmailForm, Entry, OPTION_TYPE_CHOICES

from .util import ChoiceEnum
from . import settings as local_settings


def _get_remote_ip(request):
    forwarded = request.META.get('HTTP_X_FORWARDED_FOR')
    if forwarded:
        return forwarded.split(',')[-1].strip()
    return request.META['REMOTE_ADDR']


def _get_emailform_or_404(request, slug):
    mgr = EmailForm.objects if request.user.is_staff else EmailForm.live
    return get_object_or_404(mgr, slug=slug, site=settings.SITE_ID)


def _emailform_submit(request, emailform, embed=False):
    forms = forms_for_emailform(emailform, request)
    if all(form.is_valid() for form in forms):
        all_fields = []
        for field in forms:
            for k, v in field.clean().iteritems():
                if isinstance(field, BaseOptionAnswer):
                    v = ','.join(v)
                all_fields.append('%s : %s' % (field.question.label, v))
        entry = Entry(emailform=emailform,
                      ip_address=_get_remote_ip(request),
                      content=render_to_string(
                        ('emailform/%s_emailform_completed.html'
                         % (emailform.slug),
                        'emailform/emailform_completed.html'),
                        dict(all_fields=all_fields)))
        entry.save()
        if emailform.email:
            _send_emailform_email(request, emailform, entry)
        return _emailform_show_form(request,
                                       emailform,
                                       (),
                                       entered=True,
                                       embed=embed)
    else:
        return _emailform_show_form(request,
                                       emailform,
                                       forms,
                                       embed=embed)


def _url_for_edit(request, obj):
    view_args = (obj._meta.app_label, obj._meta.module_name,)
    try:
        edit_url = reverse("admin:%s_%s_change" % view_args, args=(obj.id,))
    except NoReverseMatch:
        edit_url = "/admin/%s/%s/%d/" % (view_args + (obj.id,))
    admin_url = local_settings.EMAILFORM_ADMIN_SITE
    if not admin_url:
        admin_url = "http://" + request.META["HTTP_HOST"]
    elif len(admin_url) < 4 or admin_url[:4].lower() != "http":
        admin_url = "http://" + admin_url
    return admin_url + edit_url


def _send_emailform_email(request, emailform, entry):
    subject = emailform.title
    sender = local_settings.EMAILFORM_EMAIL_FROM
    recipients = list(set(x.strip() for x in emailform.email.split(',')))
    links = [(_url_for_edit(request, entry), "Edit Submission"),
             (_url_for_edit(request, emailform), "Edit Email Form"), ]
    html_email = entry.content
    email_msg = EmailMultiAlternatives(subject,
                                       html_email,
                                       sender,
                                       recipients)
    email_msg.attach_alternative(html_email, 'text/html')
    try:
        email_msg.send()
    except smtplib.SMTPException as ex:
        logging.exception("SMTP error sending email: %s" % str(ex))
    except Exception as ex:
        logging.exception("Unexpected error sending email: %s" % str(ex))


def _emailform_show_form(request,
                            emailform,
                            forms,
                            entered=False,
                            embed=False):
    if embed:
        specific_template = ('emailform/%s_emailform_detail_embed.html' %
                        emailform.slug)
        return render_to_response([specific_template,
                               'emailform/emailform_detail_embed.html'],
                              dict(emailform=emailform,
                                   forms=forms,
                                   entered=entered,
                                   embed=embed),
                              _rc(request))
    specific_template = ('emailform/%s_emailform_detail.html' %
                    emailform.slug)
    return render_to_response([specific_template,
                           'emailform/emailform_detail.html'],
                          dict(emailform=emailform,
                               entered=entered,
                               forms=forms,
                               embed=embed),
                          _rc(request))


def emailform_detail(request, slug, embed=False):
    emailform = _get_emailform_or_404(request, slug)
    if request.method == 'POST':
        return _emailform_submit(request, emailform, embed=embed)
    forms = forms_for_emailform(emailform, request)
    return _emailform_show_form(request, emailform, forms, embed=embed)



import re

from django.conf import settings as _gs


EMAILFORM_EMAIL_FROM = getattr(_gs,
                            'EMAILFORM_EMAIL_FROM',
                            'donotreply@donotreply.com')


EMAILFORM_ADMIN_SITE = getattr(_gs, 'EMAILFORM_ADMIN_SITE', '')



import itertools
import re

from django.utils.importlib import import_module


class ChoiceEnum(object):
    def __init__(self, choices):
        if isinstance(choices, basestring):
            choices=choices.split()
        if isinstance(choices, (list,tuple)) and all(isinstance(x, tuple) and len(x)==2 for x in choices):
            values=choices
        else:
            values=zip(itertools.count(1), choices)
        for v, n in values:
            name=re.sub('[- ]', '_', n.upper())
            setattr(self, name, v)
        self._choices=values

    def __getitem__(self, idx):
        return self._choices[idx]



from __future__ import absolute_import

import re

from django.conf import settings
from django.contrib.localflavor.us.forms import (
    USPhoneNumberField,
    USStateField,
    USZipCodeField)
from django.forms import (
    BooleanField,
    CharField,
    CheckboxSelectMultiple,
    ChoiceField,
    EmailField,
    FloatField,
    Form,
    FileField,
    IntegerField,
    MultipleChoiceField,
    RadioSelect,
    Select,
    Textarea,
    ValidationError,
    )
from django.forms.forms import BoundField

from .models import OPTION_TYPE_CHOICES


class BaseAnswerForm(Form):

    def __init__(self,
                 question,
                 entry=None,
                 *args,
                 **kwargs):
        self.question = question
        self.entry = entry
        super(BaseAnswerForm, self).__init__(*args, **kwargs)
        self._configure_answer_field()

    def as_template(self):
        """Helper function for fieldsting fields data from form."""
        bound_fields = [BoundField(self, field, name) \
                      for name, field in self.fields.items()]
        c = Context(dict(form=self, bound_fields=bound_fields))
        t = loader.get_template('emailform/form.html')
        return t.render(c)

    def _configure_answer_field(self):
        answer = self.fields['answer']
        q = self.question
        answer.required = q.required
        answer.label = q.question
        answer.help_text = q.help_text
        return answer


class USPhoneNumberAnswer(BaseAnswerForm):
    answer = USPhoneNumberField()


class USStateAnswer(BaseAnswerForm):
    answer = USStateField()


class USZipCodeAnswer(BaseAnswerForm):
    answer = USZipCodeField()


class TextInputAnswer(BaseAnswerForm):
    answer = CharField()


class IntegerInputAnswer(BaseAnswerForm):
    answer = IntegerField()


class FloatInputAnswer(BaseAnswerForm):
    answer = FloatField()


class BooleanInputAnswer(BaseAnswerForm):
    answer = BooleanField(initial=False)

    def clean_answer(self):
        value = self.cleaned_data['answer']
        if not value:
            return False
        return value


class TextAreaAnswer(BaseAnswerForm):
    answer = CharField(widget=Textarea)


class EmailAnswer(BaseAnswerForm):
    answer = EmailField()


class FileUpload(BaseAnswerForm):
    answer = FileField()


class BaseOptionAnswer(BaseAnswerForm):

    def __init__(self, *args, **kwargs):
        super(BaseOptionAnswer, self).__init__(*args, **kwargs)
        choices = [(x, x) for x in self.question.parsed_options]
        if not self.question.required:
            choices = [('', '---------',)] + choices
        self.fields['answer'].choices = choices

    def clean_answer(self):
        key = self.cleaned_data['answer']
        if not key and self.fields['answer'].required:
            raise ValidationError, _('This field is required.')
        if not isinstance(key, (list, tuple)):
            key = (key,)
        return key


class OptionAnswer(BaseOptionAnswer):
    answer = ChoiceField()


class OptionRadio(BaseOptionAnswer):
    answer = ChoiceField(widget=RadioSelect)


class OptionCheckbox(BaseOptionAnswer):
    answer = MultipleChoiceField(widget=CheckboxSelectMultiple)


## each question gets a form with one element, determined by the type
## for the answer.
QTYPE_FORM = {
    OPTION_TYPE_CHOICES.TEXT_FIELD:        TextInputAnswer,
    OPTION_TYPE_CHOICES.INTEGER:           IntegerInputAnswer,
    OPTION_TYPE_CHOICES.FLOAT:             FloatInputAnswer,
    OPTION_TYPE_CHOICES.BOOLEAN:           BooleanInputAnswer,
    OPTION_TYPE_CHOICES.TEXT_AREA:         TextAreaAnswer,
    OPTION_TYPE_CHOICES.SELECT_ONE_CHOICE: OptionAnswer,
    OPTION_TYPE_CHOICES.RADIO_LIST:        OptionRadio,
    OPTION_TYPE_CHOICES.CHECKBOX_LIST:     OptionCheckbox,
    OPTION_TYPE_CHOICES.EMAIL_FIELD:       EmailAnswer,
    OPTION_TYPE_CHOICES.FILE_UPLOAD:       FileUpload,
    OPTION_TYPE_CHOICES.PHONE_NUMBER:      USPhoneNumberAnswer,
    OPTION_TYPE_CHOICES.US_STATE:          USStateAnswer,
    OPTION_TYPE_CHOICES.ZIPCODE:           USZipCodeAnswer}


def forms_for_emailform(emailform, request='testing', entry=None):
    testing = 'testing' == request
    post = None if testing else request.POST or None
    files = None if testing else request.FILES or None
    return [_form_for_question(q, entry, post, files)
        for q in emailform.questions.all().order_by("order")]


def _form_for_question(question,
                       entry=None,
                       data=None,
                       files=None):
    return QTYPE_FORM[question.option_type](
        question=question,
        entry=entry,
        prefix='%s_%s' % (question.emailform.id,
                        question.id),
        data=data,
        files=files)



from distutils.core import setup
import os

readme_file = os.path.join(os.path.dirname(__file__),
                           'README')
long_description = open(readme_file).read()    
setup(name='django-emailform',
      version='0.1.4',
      description='django-email forms is an application for generating multi input forms, customized using the django admin, with emailing of entries to a supplied list of email addresses.',
      long_description=long_description,
      author='Glenn Mohre',
      author_email='glenn.mohre@gmail.com',
      url='http://www.bitbucket.org/glennmohre/django-emailform',
      packages=['emailform'],
      license='MIT',
     )
